% arfit2interpolate()-- takes epoched EEGLAB data structure. Interpolates
%                       single-trial data specified with interpStart and
%                       interpEnd in epoch latency (in datapoints, not seconds). 
%                       Learns ARfit model between 1 to interpStart of epoch latency.
%                       It detrends the data for internal process but put
%                       it back before output. AFfit was written by 
%
% Usage: 
%         >> output = arfit2interpolate(EEG.data(:,:,:), [interpStart interpEnd], overlapForSmoothBlending);
%
% [interpStart interpEnd]:  must be in frame, not millisecond
% overlapForSmoothBlending: the LAST n datapoints from interpEnd to be smoothly blended with the original signal 

% History:
% 01/29/2015 ver 2.1 by Makoto. SBC model order statistics display.
% 06/13/2014 ver 2.0 by Makoto. Blending implemented.
% 02/28/2014 ver 1.0 by Makoto. Created.

% Copyright (C) 2011, 2014 Makoto Miyakoshi and Tim Mullen, JSPS/SCCN,INC,UCSD.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA

function processedData = arfit2interpolate(epochedData, startEndIdx, blendingLength)

% median subtraction
meanValue = mean(epochedData(:,1:startEndIdx(1)-1,:),2);
epochedData_demean = bsxfun(@minus, epochedData, meanValue);

% linear blending
if any(blendingLength)
    blendingCurve = linspace(1,0,blendingLength);
end

processedData = zeros(size(epochedData_demean));
% main process
for chIdx = 1:size(epochedData_demean,1)
    modelOrderList = zeros(size(epochedData_demean,3),1);
    for epochIdx = 1:size(epochedData_demean,3)
        
        % single epoch
        tmpEpoch      = epochedData_demean(chIdx,:,epochIdx);
        originalEpoch = tmpEpoch;
        
        % determine model oder bounds
        minModelOrder = 1;
        maxModelOrder = round((startEndIdx(2)-startEndIdx(1)));
        
        % run arfit
        [w,A,C] = arfit(tmpEpoch(1:startEndIdx(1)-1)', minModelOrder, maxModelOrder);
        modelOrderList(epochIdx,1) = length(A);
        [R,err]= chol(C);
        noise = randn(1,length(tmpEpoch));
        
        % PREDICT THE FUTURE!! (by Tim Mullen)
        for latencyIdx = startEndIdx(1):startEndIdx(2)
            tmpEpoch(latencyIdx) = A*tmpEpoch(latencyIdx-1:-1:latencyIdx-length(A))'+noise(latencyIdx);
        end
        
        % Blending
        if any(blendingLength)
            interpolatedEpochCropped = tmpEpoch(startEndIdx(1):startEndIdx(2));
            originalEpochCropped     = originalEpoch(startEndIdx(1):startEndIdx(2));
            % figure; plot(interpolatedEpochCropped); hold on; plot(originalEpochCropped ,'r')
            fadeOutData = interpolatedEpochCropped(end-length(blendingCurve)+1:end).*blendingCurve;
            fadeInData  = originalEpochCropped(end-length(blendingCurve)+1:end).*(1-blendingCurve);
            blendedData = fadeOutData + fadeInData;
            % figure; plot(fadeOutData); hold on; plot(fadeInData, 'r'); plot(blendedData, 'g')
            tmpEpoch(startEndIdx(2)-length(blendingCurve)+1:startEndIdx(2)) = blendedData;
        end
        
        % put back the processed data
        processedData(chIdx,:,epochIdx) = tmpEpoch;
    end
    disp(sprintf('%2.0d/%2.0dch done. Model order %2.1f (SD %2.1f Range %d-%d) by Schwart''s Bayesian Criterion.', chIdx, size(epochedData_demean,1), mean(modelOrderList), std(modelOrderList), min(modelOrderList), max(modelOrderList)))
end

% add back mean values
processedData = bsxfun(@plus, processedData, meanValue);